var _ = require('underscore');
var fs = require('fs');

var mainTemplatePath = '';
var encoding = 'utf-8';

function hasValidMainTemplate( callback ) {
	
	fs.exists( mainTemplatePath, function( exists ) {
		return callback( exists );
	} );
	
}

module.exports.setMainTemplatePath = function( path ) {
	mainTemplatePath = path;
};

module.exports.setEncoding = function( e ) {
	encoding = e;
};

module.exports.setupApp = function( app, views_path ) {
	
	app.engine( 'use', function( filePath, options, callback ) {
		
		if ( options.page === undefined ) options.page = {};
		options.page.path = filePath;
		
		fs.readFile( filePath, encoding, function( err, pageData ) {
			
			if ( err ) return callback( err );
			
			hasValidMainTemplate( function( exists ) {
				
				if ( exists ) {
					
					fs.readFile( mainTemplatePath, encoding, function( err, templateData ) {
							
						if ( err ) return callback( err );
						
						var compiledPage = _.template( pageData );
						if ( options.page === undefined ) options.page = {};
						options.page.content = compiledPage( options );
						
						var compiledTemplate = _.template( templateData );
						var rendered = compiledTemplate( options );
						
						return callback( null, rendered );
						
					} );
					
				} else {
					
					var compiled = _.template( pageData );
					var rendered = compiled( options );
					
					return callback( null, rendered );
					
				}
				
			} );
			
		} );
		
	} );
	
	app.set( 'views', views_path );
	app.set( 'view engine', 'use' );
	
}